@@ -2,47 +2,35 @@
 #
 #  typing.py
 """
-Common aliases for type hinting.
-
-**Data:**
-
-.. csv-table::
-	:widths: 5, 20
-
-	:data:`~.PathLike`, Type hint for objects that represent filesystem paths.
-	:data:`~.AnyNumber`, Type hint for numbers.
-	:data:`~.WrapperDescriptorType`, The type of methods of some built-in data types and base classes.
-	:data:`~.MethodWrapperType`, The type of *bound* methods of some built-in data types and base classes.
-	:data:`~.MethodDescriptorType`, The type of methods of some built-in data types.
-	:data:`~.ClassMethodDescriptorType`, The type of *unbound* class methods of some built-in data types.
-
+Various type annotation aids.
 """
 #
 #  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
-#  This program is free software; you can redistribute it and/or modify
-#  it under the terms of the GNU Lesser General Public License as published by
-#  the Free Software Foundation; either version 3 of the License, or
-#  (at your option) any later version.
+#  Permission is hereby granted, free of charge, to any person obtaining a copy
+#  of this software and associated documentation files (the "Software"), to deal
+#  in the Software without restriction, including without limitation the rights
+#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+#  copies of the Software, and to permit persons to whom the Software is
+#  furnished to do so, subject to the following conditions:
 #
-#  This program is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-#  GNU Lesser General Public License for more details.
+#  The above copyright notice and this permission notice shall be included in all
+#  copies or substantial portions of the Software.
 #
-#  You should have received a copy of the GNU Lesser General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-#  MA 02110-1301, USA.
+#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+#  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+#  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+#  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
+#  OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
 # stdlib
 import os
 import pathlib
-import typing
 from decimal import Decimal
-from json import JSONDecoder, JSONEncoder
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union
 
 # 3rd party
 from typing_extensions import Protocol, runtime_checkable
@@ -51,8 +39,11 @@ from typing_extensions import Protocol, runtime_checkable
 import domdf_python_tools
 
 if TYPE_CHECKING or domdf_python_tools.__docs:  # pragma: no cover
+	# stdlib
+	from json import JSONDecoder, JSONEncoder
+
 	# 3rd party
-	from pandas import DataFrame, Series  # type: ignore
+	from pandas import DataFrame, Series
 
 	Series.__module__ = "pandas"
 	DataFrame.__module__ = "pandas"
@@ -61,22 +52,26 @@ if TYPE_CHECKING or domdf_python_tools.__docs:  # pragma: no cover
 	JSONEncoder.__module__ = "json"
 
 #: .. versionadded:: 1.0.0
-FrameOrSeries = typing.TypeVar("FrameOrSeries", "Series", "DataFrame")
+FrameOrSeries = TypeVar("FrameOrSeries", "Series", "DataFrame")
 
 __all__ = [
 		"PathLike",
 		"PathType",
 		"AnyNumber",
-		"check_membership",
-		"JsonLibrary",
 		"WrapperDescriptorType",
 		"MethodWrapperType",
 		"MethodDescriptorType",
 		"ClassMethodDescriptorType",
+		"JsonLibrary",
 		"HasHead",
 		"String",
 		"FrameOrSeries",
 		"SupportsIndex",
+		"SupportsLessThan",
+		"SupportsLessEqual",
+		"SupportsGreaterThan",
+		"SupportsGreaterEqual",
+		"check_membership",
 		]
 
 PathLike = Union[str, pathlib.Path, os.PathLike]
@@ -86,7 +81,7 @@ Type hint for objects that represent filesystem paths.
 .. seealso:: :py:obj:`domdf_python_tools.typing.PathType`
 """
 
-PathType = typing.TypeVar("PathType", str, pathlib.Path, os.PathLike)
+PathType = TypeVar("PathType", str, pathlib.Path, os.PathLike)
 """
 Type variable for objects that represent filesystem paths.
 
@@ -119,9 +114,9 @@ def check_membership(obj: Any, type_: Union[Type, object]) -> bool:
 
 class JsonLibrary(Protocol):
 	"""
-	Type hint for functions that take a JSON serialisation-deserialisation library as an argument.
+	:class:`typing.Protocol` for libraries that implement the same API as :mod:`json`.
 
-	The library implement at least the following methods:
+	Useful for annotating functions which take a JSON serialisation-deserialisation library as an argument.
 	"""
 
 	@staticmethod
@@ -132,7 +127,7 @@ class JsonLibrary(Protocol):
 			ensure_ascii: bool = ...,
 			check_circular: bool = ...,
 			allow_nan: bool = ...,
-			cls: Optional[Type[JSONEncoder]] = ...,
+			cls: Optional[Type["JSONEncoder"]] = ...,
 			indent: Union[None, int, str] = ...,
 			separators: Optional[Tuple[str, str]] = ...,
 			default: Optional[Callable[[Any], Any]] = ...,
@@ -159,7 +154,7 @@ class JsonLibrary(Protocol):
 	def loads(
 			s: Union[str, bytes],
 			*,
-			cls: Optional[Type[JSONDecoder]] = ...,
+			cls: Optional[Type["JSONDecoder"]] = ...,
 			object_hook: Optional[Callable[[Dict[Any, Any]], Any]] = ...,
 			parse_float: Optional[Callable[[str], Any]] = ...,
 			parse_int: Optional[Callable[[str], Any]] = ...,
@@ -215,9 +210,9 @@ class HasHead(Protocol):
 	This includes :class:`pandas.DataFrame` and :class:`pandas.Series`.
 
 	.. versionadded:: 0.8.0
-	"""  # noqa D400
+	"""
 
-	def head(self: "FrameOrSeries", n: int = 5) -> "FrameOrSeries":
+	def head(self: "HasHead", n: int = 5) -> "HasHead":
 		"""
 		Return the first n rows.
 
@@ -245,5 +240,56 @@ class SupportsIndex(Protocol):
 	.. versionadded:: 2.0.0
 	"""
 
-	def __index__(self) -> int:  # pragma: no cover
-		...
+	def __index__(self) -> int: ...
+
+
+class SupportsLessThan(Protocol):
+	"""
+	:class:`typing.Protocol` for classes that support ``__lt__``.
+
+	.. versionadded:: 3.0.0
+	"""
+
+	def __lt__(self, __other: Any) -> bool:
+		"""
+		Return ``self < value``.
+		"""
+
+
+class SupportsLessEqual(Protocol):
+	"""
+	:class:`typing.Protocol` for classes that support ``__le__``.
+
+	.. versionadded:: 3.0.0
+	"""
+
+	def __le__(self, __other: Any) -> bool:
+		"""
+		Return ``self <= value``.
+		"""
+
+
+class SupportsGreaterThan(Protocol):
+	"""
+	:class:`typing.Protocol` for classes that support ``__gt__``.
+
+	.. versionadded:: 3.0.0
+	"""
+
+	def __gt__(self, __other: Any) -> bool:
+		"""
+		Return ``self > value``.
+		"""
+
+
+class SupportsGreaterEqual(Protocol):
+	"""
+	:class:`typing.Protocol` for classes that support ``__ge__``.
+
+	.. versionadded:: 3.0.0
+	"""
+
+	def __ge__(self, __other: Any) -> bool:
+		"""
+		Return ``self >= value``.
+		"""
